﻿
using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;

namespace Affine_transforms
{
	/// <summary>
	/// Interaction logic for Window1.xaml
	/// </summary>
	public partial class Window1 : Window
	{
		Point[] Vertices = new Point[4];
        
		Point PrevPoint;
        Line[] Rect = new Line[4];
		
		public Window1()
		{
			InitializeComponent();		
			
            // inicializacia
            for (int i = 0; i < 4; i++)
                Rect[i] = new Line();
           
            for (int i = 0; i < 4; i++)
            {
                Rect[i].Stroke = new SolidColorBrush(Colors.IndianRed);
                Rect[i].StrokeThickness = 2;
                Rect[i].X1 = Vertices[i].X;
                Rect[i].Y1 = Vertices[i].Y;
                Rect[i].X2 = Vertices[(i + 1) % 4].X;
                Rect[i].Y2 = Vertices[(i + 1) % 4].Y;

                g.Children.Add(Rect[i]);
            }   
		}
		
		   // vykreslovanie obdlznika
         public void DrawRectangle(Point[] vertices)
         {
            for (int i = 0; i < 4; i++)
            {                
                Rect[i].X1 = vertices[i].X;
                Rect[i].Y1 = vertices[i].Y;
                Rect[i].X2 = vertices[(i + 1) % 4].X;
                Rect[i].Y2 = vertices[(i + 1) % 4].Y;               
            }
        }

        // posunutie -- pre jednoduchost vyuzijeme zapis po suradniciach
        public Point[] Translate(Point[] vertices, Vector dir)
        {
            Point[] output = new Point[4];        
			
            for(int i = 0; i < 4; i++)
            {
                output[i] = new Point(vertices[i].X + dir.X, vertices[i].Y + dir.Y);
            }

            return output;
        }

        // skalovanie prebieha vzhladom na tazisko objektu
        // t. j. musime najprv tazisko obdlznika presunut do pociatku sur. sustavy, nasledne preskalovat a potom obdlznik presunut na povodne miesto
        public Point[] Scale(Point[] vertices, Vector factor)
        {
            Point[] output = new Point[4];

            Vector toOrigin = new Vector();
            Point center = new Point(0, 0);

            // suradnice taziska objektu (pevne dane)    
            for (int i = 0; i < 4; i++)
            {
                center.X += 0.25 *  vertices[i].X;
                center.Y += 0.25 * vertices[i].Y;
            }

            // vektor, ktory posunie tazisko do zaciatku sur. sustavy
            toOrigin.X = 0 - center.X;
            toOrigin.Y = 0 - center.Y;

            // vykoname samotne posunutie s prislusnymi parametrami
            output = Translate(vertices, toOrigin);                
            
            // nasledne kazdy vrchol obdlznika preskalujeme
            for (int i = 0; i < 4; i++)
            {
                output[i].X = output[i].X * factor.X;
                output[i].Y = output[i].Y * factor.Y;
            }

            // obdlznik posunieme spat na povodne miesto
            output = Translate(output, -toOrigin);
            
            return output;
        }

        // podobne ako v pripade skalovania, rotaciu urobime tiez vhladom na tazisko obdlznika
        public Point[] Rotate(Point[] vertices, Double angle)
        {
            Point[] output = new Point[4];

            Vector toOrigin = new Vector();
            Point center = new Point(0, 0);

            // suradnice taziska objektu (pevne dane)   
            for (int i = 0; i < 4; i++)
            {
                center.X += 0.25 * vertices[i].X;
                center.Y += 0.25 * vertices[i].Y;
            }

            // vektor, ktory posunie tazisko do zaciatku sur. sustavy
            toOrigin.X = 0 - center.X;
            toOrigin.Y = 0 - center.Y;

            // vykoname samotne posunutie s prislusnymi parametrami
            output = Translate(vertices, toOrigin);

            // jednotlive vrcholy obdlznika zrotujeme
            for (int i = 0; i < 4; i++)
            {
                output[i] = new Point(output[i].X * Math.Cos(angle) - output[i].Y * Math.Sin(angle), output[i].X * Math.Sin(angle) + output[i].Y * Math.Cos(angle));
            }
            
            // obdlznik posunieme spat na povodne miesto
            output = Translate(output, -toOrigin);

            return output;
        }

        private void g_MouseDown(object sender, MouseButtonEventArgs e)
        {            
            PrevPoint = e.GetPosition(g);
            // definujeme poziciu vrcholov obdlznika, ktory je vo vychodiskovej polohe v pripade posunutia
            Vertices[0] = new Point(PrevPoint.X - 40, PrevPoint.Y - 40);
            Vertices[1] = new Point(PrevPoint.X + 40, PrevPoint.Y - 40);
            Vertices[2] = new Point(PrevPoint.X + 40, PrevPoint.Y + 40);
            Vertices[3] = new Point(PrevPoint.X - 40, PrevPoint.Y + 40);
        }

        private void g_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.LeftButton == MouseButtonState.Pressed)
            {
                if (rbTranslate.IsChecked == true)
                {
					Vector Dir = new Vector();
                    // vypocet vektoru posunutia
                    Dir = e.GetPosition(g) - PrevPoint;  
                    // zavolanie transformacie
                    Vertices = Translate(Vertices, Dir);
                    //vykreslenie
                    DrawRectangle(Vertices);
                    // prepiseme poziciu predosleho bodu, aby sme vedeli opat vypocitat vektor posunutia
                    PrevPoint = e.GetPosition(g);
                }

                if (rbScale.IsChecked == true)
                {
                    // v pripade skalovania budeme pracovat s pevne danym obdlznikom
                    Vertices[0] = new Point(183, 150);
                    Vertices[1] = new Point(203, 150);
                    Vertices[2] = new Point(203, 170);
                    Vertices[3] = new Point(183, 170);

                    Vector Factor = new Vector();
                    // koeficienty skalovania pocitame vzhladom na pevne tazisko obdlznika (kedze nechceme aby sa obdlznik posuval)
					PrevPoint =  new Point(193, 160);  
                    Factor = new Vector(1, 1) + (e.GetPosition(g) - PrevPoint)*0.1;
            		
                    Vertices = Scale(Vertices, Factor);
                    DrawRectangle(Vertices);
                }

                if (rbRotate.IsChecked == true)
                {
                    // aj v pripade otacania budeme pracovat s pevne danym obdlznikom
                    Vertices[0] = new Point(163, 150);
                    Vertices[1] = new Point(223, 150);
                    Vertices[2] = new Point(223, 170);
                    Vertices[3] = new Point(163, 170);

                    Double Angle;
                    Point Center = new Point(193, 160);
                    // uhol pocitame z polarnych suradnic
                    Vector u = new Vector();
                    u = e.GetPosition(g) - Center;
                    
                    if (u.Length != 0)
                    {
                        Angle = Math.Atan(u.Y /u.X);
                    }
                    else Angle = 0;

                    Vertices = Rotate(Vertices, Angle);
                    DrawRectangle(Vertices);
                    PrevPoint = e.GetPosition(g);
                }
            }
        }

        private void g_MouseUp(object sender, MouseButtonEventArgs e)
        {
           
        }
	}
}